from django.conf import settings
from django.contrib.auth.models import Group, Permission
from django.core.management.base import BaseCommand
from django.db.models import Q, QuerySet

PERMISSIONS = {
    # Program Managers get all permissions, they don’t need the edit the permissions
    settings.PRIVILEGED_GROUP: {
        "all": Permission.objects.filter(
            ~Q(codename__startswith="edit"),
            content_type__app_label="program",
        ),
    },
    # Host
    settings.ENTITLED_GROUPS[0]: {
        "default add/change note & notelink": Permission.objects.filter(
            codename__in=[
                "add_note",
                "add_notelink",
                "change_note",
                "change_notelink",
            ],
        ),
        "default change profile": Permission.objects.filter(codename="change_profile"),
        "custom add media-source": Permission.objects.filter(
            codename__in=[
                "add__file",
                "add__import",
            ]
        ),
        "custom edit note": Permission.objects.filter(
            ~Q(codename="edit__note__topics"),
            ~Q(codename="edit__note__languages"),
            codename__startswith="edit__note",
        ),
        "custom edit profile": Permission.objects.filter(codename="edit__profile__name"),
    },
    # Host+
    settings.ENTITLED_GROUPS[1]: {
        "default add/change note & notelink": Permission.objects.filter(
            codename__in=[
                "add_note",
                "add_notelink",
                "change_note",
                "change_notelink",
            ],
        ),
        "default change profile, schedule & show": Permission.objects.filter(
            codename__in=[
                "change_profile",
                "change_schedule",
                "change_show",
            ],
        ),
        "custom add media-source": Permission.objects.filter(
            codename__in=[
                "add__file",
                "add__import",
                "add__line",
                "add__stream",
            ]
        ),
        "custom edit note": Permission.objects.filter(
            ~Q(codename="edit__note__topics"),
            codename__startswith="edit__note",
        ),
        "custom edit profile": Permission.objects.filter(
            codename__in=[
                "edit__profile__biography",
                "edit__profile__email",
                "edit__profile__image",
                "edit__profile__links",
                "edit__profile__name",
            ]
        ),
        "custom edit schedule": Permission.objects.filter(
            codename="edit__schedule__default_playlist_id"
        ),
        "custom edit show": Permission.objects.filter(
            codename__in=[
                "edit__show__default_playlist_id",
                "edit__show__description",
                "edit__show__email",
                "edit__show__hosts",
                "edit__show__image",
                "edit__show__links",
                "edit__show__logo",
                "edit__show__short_description",
            ]
        ),
    },
}


class Command(BaseCommand):
    help = "add permissions"

    def add_permissions(self, group: Group, permissions: QuerySet, name: str):
        self.stdout.write(f"Adding {name} permissions to the '{group.name}' group... ", ending="")
        self.stdout.flush()
        group.permissions.add(*permissions)
        self.stdout.write(self.style.SUCCESS(str(len(permissions))))

    def handle(self, *args, **options):
        for group_name in PERMISSIONS:
            group = Group.objects.get(name=group_name)

            for name, permissions in PERMISSIONS[group_name].items():
                self.add_permissions(group, permissions, name)
